@@ -15,7 +15,7 @@ try:
 except ImportError:
     import json
 
-import hypercouch.index
+from hypercouch.index import Index
 
 def options():
     return [
@@ -28,7 +28,7 @@ def options():
 def requests():
     line = sys.stdin.readline()
     while line:
-        #sys.stderr.write("%s\n\n" % line.strip())
+        line = line.decode("utf-8")
         yield json.loads(line)
         line = sys.stdin.readline()
 
@@ -39,7 +39,7 @@ def main():
         print "Unknown arguments: %s" % ' '.join(args)
         parser.print_help()
         exit(-1)
-    idx = hypercouch.index.Index(opts.dir, opts.uri)
+    idx = Index(opts.dir, opts.uri)
     try:
         for req in requests():
             try: 
