@@ -37,10 +37,8 @@ class Index(object):
         self.hdb = hypy.HDatabase()
         self.hdb.open(self.idxdir, "a")
         self.couch = couchdb.Server(uri)
-        self.rt = spidermonkey.Runtime()
-        self.cx = self.rt.create_context()
-        self.cx.bind(u"index", self.add_text)
-        self.cx.bind(u"property", self.add_attribute)
+        self.rt = None
+        self.cx = None
         self.lock = threading.RLock()
         self.comitted = False
         self.closing = False
@@ -50,7 +48,6 @@ class Index(object):
         self.curr_doc = None
         self.timeout = timeout
         self.indexer = threading.Thread(target=self.index)
-        self.indexer.setDaemon(False)
         self.indexer.start()
         self.do_notify = os.environ.get("HYPERCOUCH_NOTIFY", "true") != "false"
 
@@ -116,6 +113,12 @@ class Index(object):
         return (ret, highlight)
 
     def index(self):
+        assert self.rt is None and self.cx is None
+        self.rt = spidermonkey.Runtime()
+        self.cx = self.rt.new_context()
+        self.cx.add_global("index", self.add_text)
+        self.cx.add_global("property", self.add_attribute)
+        
         self.read_state()
         while True:
             try:
@@ -137,31 +140,38 @@ class Index(object):
                     continue
                 # else update required
 
-                updates = db.view("_all_docs_by_seq", startkey=curr_seq, limit=500)
+                updates = db.view("_all_docs_by_seq",
+                                    startkey=curr_seq, limit=500)
                 while len(updates) > 0:
                     docids = []
                     processed = {}
                     for row in updates.rows:
                         docids.append(row.id)
-                    for row in db.view("_all_docs", wrapper=view_wrapper, keys=docids, include_docs=True):
+                    for row in db.view("_all_docs",
+                                            keys=docids, include_docs=True):
                         if row.id.startswith("_design/"):
-                            # Check if we deleted a design doc that had an index function
-                            if row.value.get("deleted") and row.id in self.state[dbname]["functions"]:
-                                raise PurgeIndexRequired(dbname, "Design document deleted.")
+                            # Check if we deleted a design doc that had
+                            # an index function
+                            if row.value.get("deleted") \
+                                and row.id in self.state[dbname]["functions"]:
+                                raise PurgeIndexRequired(dbname,
+                                                "Design document deleted.")
                             elif row.value.get("deleted"):
                                 # A design doc we don't care about was deleted.
                                 continue
                             # Check that ft_index is the same as we have
                             curr = self.state[dbname]["functions"].get(row.id)
                             if curr != row.doc.get("ft_index"):
-                                raise PurgeIndexRequired(dbname, "Design ft_index changed.")
+                                raise PurgeIndexRequired(dbname,
+                                                    "Design ft_index changed.")
                             # Don't index design docs.
                             continue
                         if row.value.get("deleted"):
                             processed[row.id] = None
                             continue
                         try:
-                            self.curr_doc = hypy.HDocument(uri=self.mk_uri(dbname, row.id))
+                            self.curr_doc = hypy.HDocument(
+                                            uri=self.mk_uri(dbname, row.id))
                             try:
                                 for ddoc, fn in funcs.iteritems():
                                     fn(row.doc)
@@ -171,7 +181,10 @@ class Index(object):
                             except:
                                 traceback.print_exc()
                         except UnicodeEncodeError:
-                            self.notify("Ignoring unicode error on document: /%s/%s" % (dbname, row.id))
+                            self.notify(
+                                "Ignoring unicode error on document: /%s/%s" \
+                                                        % (dbname, row.id)
+                            )
                     curr_seq = updates.rows[-1]["key"]
                     with self.lock:
                         if self.closing:
@@ -185,7 +198,8 @@ class Index(object):
                         self.state[dbname]["update_seq"] = curr_seq
                         self.write_state()
                         self.comitted = True
-                    updates = db.view("_all_docs_by_seq", startkey=curr_seq, limit=500)
+                    updates = db.view("_all_docs_by_seq",
+                                                startkey=curr_seq, limit=500)
             except PurgeIndexRequired, inst:
                 self.notify("PURGING '%s': %s" % (inst.dbname, inst.mesg))
                 self.purge(inst.dbname)
@@ -194,7 +208,10 @@ class Index(object):
                 self.purge(dbname)
             # Indexing thread exiting.
             except Exception, inst:
-                self.notify("Uncaught exception in indexing thread, shutting down forcefully.")
+                self.notify(
+                        "Uncaught exception in indexing thread, " \
+                        "shutting down forcefully."
+                )
                 traceback.print_exc()
                 os._exit(-1)
 
@@ -203,17 +220,18 @@ class Index(object):
         update_seq = 0
         if "functions" in self.state.get(dbname, {}):
             for k, v in self.state[dbname]["functions"].iteritems():
-                funcs[k] = self.cx.execute(self.mk_unicode(v))
+                funcs[k] = self.cx.execute(v)
             if len(funcs) > 0:
                 return (funcs, self.state[dbname]["update_seq"])
         self.state.setdefault(dbname, {})
         self.state[dbname].setdefault("update_seq", 0)
         self.state[dbname].setdefault("functions", {})
-        for row in db.view("_all_docs", startkey="_design/", endkey="_design0", include_docs=True):
+        for row in db.view("_all_docs",
+                    startkey="_design/", endkey="_design0", include_docs=True):
             if "ft_index" in row.doc:
                 self.state[dbname]["functions"][row.id] = row.doc["ft_index"]
                 script = row.doc["ft_index"]
-                funcs[row.id] = self.cx.execute(self.mk_unicode(script))
+                funcs[row.id] = self.cx.execute(script)
         sig = self.state[dbname]["update_seq"]
         return (funcs, update_seq)
 
@@ -243,25 +261,33 @@ class Index(object):
     def mk_uri(self, dbname, docid):
         return u"/%s/%s" % (urllib.quote(dbname), urllib.quote(docid))
 
-    def mk_unicode(self, data):
-        if not isinstance(data, types.StringTypes):
-            data = str(data)
-        if not isinstance(data, types.UnicodeType):
-            return data.decode("utf-8")
-        return data
+    def mk_unicode(self, val):
+        if not isinstance(val, basestring):
+            return unicode(val)
+        if isinstance(val, unicode):
+            return val
+        return val.decode("utf-8")
 
     def notify(self, mesg):
         if self.do_notify:
             sys.stderr.write("%s\n" % mesg)
 
     def add_text(self, text):
-        text = self.mk_unicode(text)
-        self.curr_doc.addText(text)
+        try:
+            text = self.mk_unicode(text)
+            self.curr_doc.addText(text)
+        except:
+            traceback.print_exc()
+            raise
 
     def add_attribute(self, key, value):
-        key = self.mk_unicode(key)
-        value = self.mk_unicode(value)
-        self.curr_doc[key] = value
+        try:
+            key = self.mk_unicode(key)
+            value = self.mk_unicode(value)
+            self.curr_doc[key] = value
+        except:
+            traceback.print_exc()
+            raise
 
 if __name__ == '__main__':
     main()
