@@ -13,9 +13,8 @@ class UnicodeTest(common.Common):
                 docs.append({"_id": str(i), "body": u"This is doc: \u00FC", "foo": u"\u00FC"})
             else:
                 docs.append({"_id": str(i), "body": "This is doc foo.", "foo": u"\u00FC"})
-        print docs
         self.db.update(docs)
-        self.wait(expect=10, display=True)
+        self.wait(expect=10)
         data = self.query(q="*.**")
         self.assertEqual(len(data["rows"]), 10)
         for doc in data["rows"]:
